@@ -2,7 +2,11 @@ import os
 from .settings import *
 
 INSTALLED_APPS = [
+    'django.contrib.auth',
+    'django.contrib.admin',
     'django.contrib.contenttypes',
+    'django.contrib.messages',
+    'haystack',
     'test_haystack.core',
     'test_haystack.xapian_tests',
 ]
