@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import datetime
 
 from django.conf import settings
@@ -27,7 +25,7 @@ class XapianSearchQueryTestCase(HaystackBackendTestCase, TestCase):
         return MockQueryIndex()
 
     def setUp(self):
-        super(XapianSearchQueryTestCase, self).setUp()
+        super().setUp()
         self.sq = connections['default'].get_query()
 
     def test_all(self):
@@ -72,17 +70,12 @@ class XapianSearchQueryTestCase(HaystackBackendTestCase, TestCase):
         self.sq.add_filter(SQ(content=datetime.datetime(2009, 5, 8, 11, 28)))
         self.assertExpectedQuery(self.sq.build_query(),
                                  '((Z2009-05-08 OR 2009-05-08) OR'
-                                 ' (Z11:28:00 OR 11:28:00))',
-                                 xapian12string='(Z2009-05-08 OR 2009-05-08 OR'
-                                                ' Z11:28:00 OR 11:28:00)')
+                                 ' (Z11:28:00 OR 11:28:00))')
 
     def test_datetime_not(self):
         self.sq.add_filter(~SQ(content=datetime.datetime(2009, 5, 8, 11, 28)))
         self.assertExpectedQuery(self.sq.build_query(),
-                                 '(<alldocuments> AND_NOT ((Z2009-05-08 OR 2009-05-08) OR (Z11:28:00 OR 11:28:00)))',
-                                 xapian12string='(<alldocuments> AND_NOT '
-                                                '(Z2009-05-08 OR 2009-05-08 OR'
-                                                ' Z11:28:00 OR 11:28:00))')
+                                 '(<alldocuments> AND_NOT ((Z2009-05-08 OR 2009-05-08) OR (Z11:28:00 OR 11:28:00)))')
 
     def test_float(self):
         self.sq.add_filter(SQ(content=25.52))
@@ -105,8 +98,7 @@ class XapianSearchQueryTestCase(HaystackBackendTestCase, TestCase):
         self.sq.add_filter(SQ(content='hello') | SQ(content='world'))
         self.assertExpectedQuery(
             self.sq.build_query(),
-            '((Zhello OR hello) OR (Zworld OR world))',
-            xapian12string='(Zhello OR hello OR Zworld OR world)')
+            '((Zhello OR hello) OR (Zworld OR world))')
 
     def test_multiple_words_or_not(self):
         self.sq.add_filter(~SQ(content='hello') | ~SQ(content='world'))
@@ -120,9 +112,7 @@ class XapianSearchQueryTestCase(HaystackBackendTestCase, TestCase):
         self.assertExpectedQuery(
             self.sq.build_query(),
             '(((Zwhi OR why) OR (Zhello OR hello)) AND '
-            '(<alldocuments> AND_NOT (Zworld OR world)))',
-            xapian12string='((Zwhi OR why OR Zhello OR hello) AND'
-                           ' (<alldocuments> AND_NOT (Zworld OR world)))',)
+            '(<alldocuments> AND_NOT (Zworld OR world)))')
 
     def test_multiple_word_field_exact(self):
         self.sq.add_filter(SQ(foo='hello'))
@@ -141,15 +131,13 @@ class XapianSearchQueryTestCase(HaystackBackendTestCase, TestCase):
     def test_or(self):
         self.sq.add_filter(SQ(content='hello world'))
         self.assertExpectedQuery(
-            self.sq.build_query(), '((Zhello OR hello) OR (Zworld OR world))',
-            xapian12string='(Zhello OR hello OR Zworld OR world)')
+            self.sq.build_query(), '((Zhello OR hello) OR (Zworld OR world))')
 
     def test_not_or(self):
         self.sq.add_filter(~SQ(content='hello world'))
         self.assertExpectedQuery(
             self.sq.build_query(),
-            '(<alldocuments> AND_NOT ((Zhello OR hello) OR (Zworld OR world)))',
-            xapian12string='(<alldocuments> AND_NOT (Zhello OR hello OR Zworld OR world))')
+            '(<alldocuments> AND_NOT ((Zhello OR hello) OR (Zworld OR world)))')
 
     def test_boost(self):
         self.sq.add_filter(SQ(content='hello'))
@@ -248,6 +236,13 @@ class XapianSearchQueryTestCase(HaystackBackendTestCase, TestCase):
         self.sq.add_filter(SQ(django_ct='time'))
         self.assertExpectedQuery(self.sq.build_query(), 'CONTENTTYPEtime')
 
+    def test_unphrased_id(self):
+        'An internal ID should NOT be phrased so one can exclude IDs.'
+        self.sq.add_filter(SQ(id__in=['testing123', 'testing456']))
+        expected = '(Qtesting123 OR Qtesting456)'
+        self.assertExpectedQuery(
+            query=self.sq.build_query(), string_or_list=expected)
+
 
 class SearchQueryTestCase(HaystackBackendTestCase, TestCase):
     """
@@ -260,7 +255,7 @@ class SearchQueryTestCase(HaystackBackendTestCase, TestCase):
         return MockSearchIndex()
 
     def setUp(self):
-        super(SearchQueryTestCase, self).setUp()
+        super().setUp()
 
         self.backend.update(self.index, MockModel.objects.all())
 
@@ -381,7 +376,7 @@ class LiveSearchQuerySetTestCase(HaystackBackendTestCase, TestCase):
         return MockSearchIndex()
 
     def setUp(self):
-        super(LiveSearchQuerySetTestCase, self).setUp()
+        super().setUp()
 
         self.backend.update(self.index, MockModel.objects.all())
         self.sq = connections['default'].get_query()
@@ -413,7 +408,7 @@ class BoostFieldTestCase(HaystackBackendTestCase, TestCase):
         return BoostMockSearchIndex()
 
     def setUp(self):
-        super(BoostFieldTestCase, self).setUp()
+        super().setUp()
 
         self.sample_objs = []
         for i in range(1, 5):
